import os
from dataclasses import dataclass

import ldap

from univention.admin.uldap import access


def get_ldap_base_from_traeger(hostname: str) -> str:
    ldap_con = ldap.initialize(f"ldap://{hostname}:7389")
    ncs = ldap_con.get_naming_contexts()
    ncs.remove(b"cn=translog")
    return ncs[0].decode("UTF-8")


def get_traeger_ldap_con(traeger: str) -> access:
    """
    UCS_ENV_TRAEGER1_DOMAIN and UCS_ENV_TRAEGER2_DOMAIN
    needs to be set by the (jenkins) job
    dito for the /var/lib/ucs/traeger1[2].secret files
    """
    traegers = {
        "Traeger1": f"ucs-sso.{os.environ.get('UCS_ENV_TRAEGER1_DOMAIN')}",
        "Traeger2": f"ucs-sso.{os.environ.get('UCS_ENV_TRAEGER2_DOMAIN')}",
    }
    hostname = traegers[traeger]
    password = open(f"/var/lib/ucs-test/{traeger}.secret").read().strip()
    ldap_base = get_ldap_base_from_traeger(hostname)
    lo = access(
        hostname,
        start_tls=1,
        base=ldap_base,
        binddn=f"uid=Administrator,cn=users,{ldap_base}",
        bindpw=password,
    )
    return lo


@dataclass
class User:
    username: str
    firstname: str
    lastname: str
    displayName: str
    ucsschoolRole: list
    dn: str
    entryUUID: str


def get_user_from_traeger(traeger: str, filter_l: str) -> User:
    for dn, attrs in get_traeger_ldap_con(traeger).search(filter_l, attr=["*", "+"]):
        yield User(
            username=attrs.get("uid", [b""])[0].decode("utf-8"),
            firstname=attrs.get("givenName", [b""])[0].decode("utf-8"),
            lastname=attrs.get("sn", [b""])[0].decode("utf-8"),
            displayName=attrs.get("gecos", [b""])[0].decode("utf-8"),
            ucsschoolRole=[role.decode("utf-8") for role in attrs.get("ucsschoolRole", [b""])],
            dn=dn,
            entryUUID=attrs.get("entryUUID", [b""])[0].decode("utf-8"),
        )
